#include <iostream>
#include <cstdarg>
#include <vector>

using namespace std;

#include "fraction"

template<class sourcetype, class tmptype=fraction<sourcetype> >
class algofinder
{
private:
    struct elem
    {
        tmptype integer;        /* Number this is multiplied by      */
        unsigned exponent;  /* 0=constant, 1=x, 2=xx, 3=xxx, ... */
    public:
        elem() : integer(0), exponent(0) { }
        elem(int add, unsigned exp) : integer(add), exponent(exp) { }
    };
    
    bool Test(
        const vector<sourcetype> &keys,
        const vector<sourcetype> &values,
        const vector<algofinder<sourcetype,tmptype>::elem> &guess) const
    {
        bool wasok = true;
        
        /* Test the guess */

        for(unsigned a=0; a<keys.size(); ++a)
        {
            tmptype val = keys[a];
            tmptype result = 0;
                
            for(unsigned b=0; b<guess.size(); ++b)
            {
                tmptype thisval = 1;
                for(unsigned c=0; c<guess[b].exponent; ++c)thisval *= val;
                thisval *= guess[b].integer;
                result += thisval;
            }
            if(result != (tmptype)values[a])
            {
                /* Wrong */
                //wasok = false;
                //cerr << "  Nope, " << result.value() << " != " << values[a] << endl;
                //goto Ouch;
                return false;
            }
        }
        cerr << "\nFound:  " << flush;

    //Ouch:
        for(unsigned b=0; b<guess.size(); ++b)
        {
            if(guess[b].integer)
            {
            	bool nega = guess[b].integer.negative();
            	
                if(b)cout << ' ';
                
                sourcetype nom = guess[b].integer.nom();
                sourcetype denom=guess[b].integer.denom();
                
                if(nega ^ (nom < 0)) { nom=-nom; denom=-denom; }
                
                if(nom == -1 && guess[b].exponent)
                {
                    cout << '-';
                    if(denom != 1)cout << ' ';
                }
                else
                {
                    if(b && nom >= 0)cout << '+';
                    if(abs(nom) > 1 || !guess[b].exponent)cout << nom;
                }
                if(guess[b].exponent)
                {
                    cout << 'x';
                    if(guess[b].exponent==2)
                        cout << '';
                    else if(guess[b].exponent > 1)
                        cout << '^' << guess[b].exponent;
                }
                if(denom != 1)
                    cout << '/' << denom;
            }
        }
        cout << endl;
        
        if(wasok) { cerr << endl; }
        
        return wasok;
    }

    tmptype calcdet(const vector<sourcetype> &mat, unsigned depf) const
    {
        tmptype sum;
        if(depf <= 1)
            sum = mat[0];
        else
        {
            sum = 0;
            for(unsigned a=0; a<depf; ++a)
            {
                unsigned subdepf = depf-1;
                vector<sourcetype> tmp(subdepf * subdepf);
                for(unsigned b=0; b<subdepf; ++b)
                    for(unsigned c=0; c<subdepf; ++c)
                        tmp[b*subdepf + c] = mat[(1+b) * depf + (c + (c>=a))%depf];
                tmptype subdet = mat[a] * calcdet(tmp, subdepf);
                if(a & 1)
                    sum -= subdet;
                else
                    sum += subdet;
            }
        }
        return sum;
    }

public:
    void Guess(
        const vector<sourcetype> &keys,
        const vector<sourcetype> &values) const
    {
        if(keys.size() != values.size())
        {
            cerr << "Error: keys.size() != values.size()\n";
            return;
        }
        
        cerr << "Analyzing {";
        for(unsigned a=0; a<keys.size(); ++a)
        {
            if(a) cerr << ',';
            cerr << keys[a];
        }
        cerr << "} => {";
        for(unsigned a=0; a<values.size(); ++a)
        {
            if(a) cerr << ',';
            cerr << values[a];
        }
        cerr << "}\n";
        
        for(unsigned elemcount=1; elemcount<=keys.size(); ++elemcount)
        {
            vector<sourcetype> mat (elemcount * elemcount);
            
            for(unsigned a=0; a<elemcount; ++a)
                for(unsigned b=0; b<elemcount; ++b)
                {
                    sourcetype val = 1;
                    for(unsigned c=0; c<b; ++c)val *= keys[a];
                    mat[a*elemcount + b] = val;
                }
            
            tmptype det = calcdet(mat, elemcount);
            
            vector<algofinder<sourcetype,tmptype>::elem> guess(elemcount);
            
            for(unsigned a = 0; a < elemcount; ++a)
            {
                vector<sourcetype> mat2 = mat;
                
                for(unsigned c = 0; c < elemcount; ++c)
                    mat2[c*elemcount+a] = values[c];
                
                guess[elemcount-1-a].integer  = calcdet(mat2, elemcount) / det;
                guess[elemcount-1-a].exponent = a;
            }
            
            if(Test(keys, values, guess))
                return;
        }
        cerr << "Not found!\n";
    }

    vector<sourcetype> array(sourcetype a,sourcetype b,sourcetype c,sourcetype d) const
    {
        vector<sourcetype> result(4);
        result[0]=a;result[1]=b;result[2]=c;result[3]=d;
        return result;
    }

    void Test() const
    {
        /* Should output: x + 4 */     
        
        Guess( array(0,1,2,3),           /*    x*0 + y*1 = 4      x=(4*1-0*5) / (1-0)=4 */
               array(4,5,6,7) );         /*    x*1 + y*1 = 5      y=(5  -  4) / (1-0)=1 */
        
        /* Should output: 2x */

        Guess( array(10,11,12,13),       /*    x*10 + y*1 = 20    x=(20*11-22*10) / (11-10)=0 */
               array(20,22,24,26) );     /*    x*11 + y*1 = 22    y=(22   -20)    / (11-10)=2 */
        
        /* Should output: 3x + 1 */
        
        Guess( array(1,2,3,4),
               array(4,7,10,13) );
        
       /* Should output: xx - 1 */
        
        Guess( array(1,2,3,4),
               array(0,3,8,15) );

        /* Should output: xx - 2x +20 */
        
        Guess( array( 1, 2, 3, 4),    /*  x*1 + y*1 + z*1 = 19  x = 1*(2-3)+1(-9blah  */
               array(19,20,23,28) );  /*  x*4 + y*2 + z*1 = 20  it works anyway. :) */
                                      /*  x*9 + y*3 + z*1 = 23 */
        Guess( array( 1, 2, 3, 5),
               array(11,27,23,29) );
    }
};


#include <argh.hh>

int main(void)
{
    algofinder<int> gruu;
    gruu.Test();
    
    cerr << "\n\n\n";
  
    return 0;
}
